@@ -484,8 +484,8 @@ object or the name of the stage.
 		#			$self->_croak( "Arg ($arg) may not be an array ref" )
 		#				if ref($args{$arg}) eq 'ARRAY';
 		#		}
-		my @deps     = _build_stage_list($args{dep}, $args{deps});
-		my @pre_reqs = _build_stage_list($args{pre_req}, $args{pre_reqs});
+		my @deps     = $self->_build_stage_list($args{dep}, $args{deps});
+		my @pre_reqs = $self->_build_stage_list($args{pre_req}, $args{pre_reqs});
 		for my $dep (@deps) {
 			for my $pre_req (@pre_reqs) {
 				$self->_add_dep( $pre_req, $dep );
@@ -494,7 +494,13 @@ object or the name of the stage.
 	};
 
 	sub _build_stage_list {
+		my $self = shift;
+		my $stages = $self->_stages;
 		return
+		    map  { $self->croak( "Unknown stage name ($_) passed to add_deps" ) 
+			           unless exists $stages->{$_};
+				   $_
+				 }
 			grep { defined }
 			map  { ref($_) ? $_->name : $_ }
 			map  { ref($_) eq 'ARRAY' ? @{$_} : ($_) }
@@ -522,7 +528,7 @@ other stage or the attempt is aborted).
 		my $self = shift;
 		my $catcher = sub {
 			my $sig = shift;
-			$self->info( "Killed with signal SIG$sig" );
+			$self->warn( "Killed with signal SIG$sig" );
 			$self->kill_stages;
 		};
 		local $SIG{HUP} = $catcher;
@@ -697,7 +703,7 @@ other stage or the attempt is aborted).
 		my $pre_req = shift;
 		for my $dep ( keys %{ $self->_deps->{$pre_req} } ) {
 			delete $self->_pre_reqs->{$dep}{$pre_req};
-			$self->info( "removing dep: $pre_req -> $dep, remaining deps are( "
+			$self->debug( "removing dep: $pre_req -> $dep, remaining deps are( "
 				. join( ' ', sort keys %{ $self->_pre_reqs->{$dep} } )
 				. " )" );
 			unless (keys %{ $self->_pre_reqs->{$dep} }) {
