@@ -290,7 +290,7 @@ has 'stage_dir_name' => (
     lazy    => 1,
     default => sub {
         my $self   = shift;
-		return $self->name;
+        return $self->name;
     },
 );
 
@@ -302,12 +302,12 @@ has 'stage_dir' => (
     default => sub {
         my $self   = shift;
         my $target;
-		if ($self->_has_dir) {
-			$target = $self->dir;
-		}
-		else {
-			$target = $self->group->group_dir->subdir( $self->stage_dir_name );
-		}
+        if ($self->_has_dir) {
+            $target = $self->dir;
+        }
+        else {
+            $target = $self->group->group_dir->subdir( $self->stage_dir_name );
+        }
         HPCI::_trigger_mkdir($self,$target);
         return $target;
     },
@@ -315,10 +315,10 @@ has 'stage_dir' => (
 );
 
 has 'dir' => (
-    is      => 'ro',
-    isa     => Dir,
-    coerce  => 1,
-	predicate => '_has_dir'
+    is        => 'ro',
+    isa       => Dir,
+    coerce    => 1,
+    predicate => '_has_dir'
 );
 
 =head2 resources_required
@@ -458,6 +458,109 @@ sub _croak {
     croak $msg;
 }
 
+=head2 files
+
+A hash that can contain lists of files.
+
+The top level of the hash has keys 'in' and 'out' for input and output.
+(The same file might be under both.)
+
+The next level down in each of these is 'req' and 'opt' for required and
+optional.
+
+A file may be listed under both the 'in' and 'out' sub-hashes, but at least one
+of those listings must be under 'opt'.  This would be used for files that are
+updated in place by the stage program.
+
+=over 4
+
+=item
+
+'in' => 'req' files will be needed as input to the program - (in/req)
+
+=item
+
+'in' => 'opt' files will be used as input to the program if they are present - (in/opt)
+
+=item
+
+'out' => 'req' files must be generated as output by the program - (out/req)
+
+=item
+
+'out' => 'opt' files can be generated or modified as output by the program - (out/opt)
+
+=back
+
+These lists can be used in a number of ways.
+
+=over
+
+=item 1
+
+If stages are allowed to be skipped when unnecessary, the decision to skip a
+stage is made by:
+
+=over 4
+
+=item
+
+all in/req and out/req files must exist
+
+=item
+
+the timestamp on the newest in/req file must be older than the timestamp on the oldest out/req file
+
+=back
+
+So, the stage will skipped if all the required inputs are older than all of the requierd
+outputs.  If a file is updated in place,, then either it should be listed as optional
+in the output list, or else the stage program must always update the other outputs after
+this one - otherwise the execution of the stage can never happen.
+
+=item 1
+
+If the cluster type is not using a global shared file system for all nodes then this
+attribute is used to control copying of files to and from the cluster node that will
+execute the stage command.
+
+=over 4
+
+=item
+
+in/req files are copied to the node that will run the stage before the stage is executed
+
+=item
+
+in/opt files are copied to the node that will run the stage if they exist before the stage is executed
+
+=item
+
+out/req files are copied back from the node after the stage has been executed
+
+=item
+
+out/opt files are copied back from the node after the stage has been executed if they exists
+(possibly not copyiny them if they have not been changed - that is cluster-specific behaviour)
+
+=back
+
+=item 1
+
+If automatic validation of stage execution is being done then
+all out/req files must exist and be timestamped later than the start of the execution
+of the stage.
+
+=back
+
+=cut
+
+has 'files' => (
+    is        => 'ro',
+    isa       => 'HashRef',
+    predicate => '_has_files',
+);
+
 =head2 state, is_ready, is_blocked, is_pass, is_fail
 
 These states are mostly for internal use during execution.  (There
@@ -533,9 +636,9 @@ to code that checks the result.
 The code will be called with the arguments:
 
     $coderef->( $stats, $stdout, $stderr )
-	    $stats is a hashref containing the accounting info and status of the job
-		$stdout is the pathname of the stdout file
-		$stderr is the pathname of the srderr file
+        $stats  is a hashref containing the accounting info and status of the job
+        $stdout is the pathname of the stdout file
+        $stderr is the pathname of the srderr file
 
 The code can use these (and knowledge provided by the user about the stage
 operation) to test whether the stage succeeded.
@@ -543,8 +646,8 @@ operation) to test whether the stage succeeded.
 The return value from the code should be a string, one of:
 
     pass
-	fail
-	retry
+    fail
+    retry
 
 to classify the result.  If the value is set to 'fail', other
 mechanisms may still apply to change that to 'retry'.
@@ -559,25 +662,41 @@ This code attribute could be used for:
 
 has 'verify_completion_state' => (
     is        => 'ro',
-	isa       => 'CodeRef',
+    isa       => 'CodeRef',
     predicate => '_has_verify_completion_state',
 );
 
 after '_analyse_completion_state' => sub {
-	my $self  = shift;
-	my $run   = shift;
+    my $self  = shift;
+    my $run   = shift;
 
-	if ($self->_has_verify_completion_state) {
-		my $stats  = $run->stats;
-		my $stdout = $run->_stdout;
-		my $stderr = $run->_stderr;
-		my $code   = $self->verify_completion_state->( $stats, $stdout, $stderr );
-		if ($code eq 'retry') {
-			$code = 'fail';
-			$run->_analysis_chose_retry(1);
-		}
-		$self->_set_state( $code ) if $code eq 'pass' || $code eq 'fail';
-	}
+    if ($self->is_pass && $self->_has_files) {
+        my $script_time = -M $self->script_file;
+        my $files = $self->files;
+        FILE:
+        for my $file (map { ref($_) ? @$_ : $_ } $files->{out}{req} ) {
+            unless (-e $file && (-M _ <= $script_time)) {
+                $self->_set_state('fail');
+                $self->error(
+                    "Stage (", $self->name,
+                    ") status changed to failed because required output file $file was not updated"
+                );
+                $run->stats->{failure_detected} = "required output file $file not updated";
+                last FILE;
+            }
+        }
+    }
+    if ($self->_has_verify_completion_state) {
+        my $stats  = $run->stats;
+        my $stdout = $run->_stdout;
+        my $stderr = $run->_stderr;
+        my $code   = $self->verify_completion_state->( $stats, $stdout, $stderr );
+        if ($code eq 'retry') {
+            $code = 'fail';
+            $run->_analysis_chose_retry(1);
+        }
+        $self->_set_state( $code ) if $code eq 'pass' || $code eq 'fail';
+    }
 };
 
 =head2 should_choose_retry, choose_retries
@@ -603,23 +722,23 @@ The default is 1.
 =cut
 
 has 'should_choose_retry' => (
-	is      => 'ro',
-	isa     => 'CodeRef',
-	predicate => '_has_should_choose_retry'
+    is        => 'ro',
+    isa       => 'CodeRef',
+    predicate => '_has_should_choose_retry'
 );
 
 has 'choose_retries' => (
-	is        => 'ro',
-	isa       => 'Int',
-	lazy      => 1,
-	default   => 1,
+    is        => 'ro',
+    isa       => 'Int',
+    lazy      => 1,
+    default   => 1,
 );
 
 has '_chosen_retries' => (
-	is        => 'rw',
-	isa       => 'Int',
+    is        => 'rw',
+    isa       => 'Int',
     init_arg  => undef,
-	default   => 0
+    default   => 0
 );
 
 =head2 force_retries
@@ -645,16 +764,16 @@ so there can be as amny retries as the total of the number allowed by each type.
 =cut
 
 has 'force_retries' => (
-	is        => 'ro',
-	isa       => 'Int',
-	default   => 0
+    is        => 'ro',
+    isa       => 'Int',
+    default   => 0
 );
 
 has '_forced_retries' => (
-	is        => 'rw',
-	isa       => 'Int',
+    is        => 'rw',
+    isa       => 'Int',
     init_arg  => undef,
-	default   => 0
+    default   => 0
 );
 
 has '_failure_info' => (
